import 'package:adeptarch/src/widgets/widgets.dart';
import 'package:adeptarch/src/widgets/sort_view.dart';
import 'package:adeptarch/src/filters/filters.dart';
import 'package:flutter/material.dart';

class NumberFilterView extends BaseFilterView<NumberFilter> {

  String textUnspecified, textExceptionValueValidation;

  NumberFilterView({
    FilterController<NumberFilter> filterController,
    String hint,
    this.textUnspecified,
    this.textExceptionValueValidation,
  }) : super(
    filterController: filterController,
    hint: hint,
  ){
    textUnspecified ??= "Unspecified";
    textExceptionValueValidation ??= "Value must be an integer number.";
  }

  @override
  State createState()  => _NumberFilterViewState();
}

class _NumberFilterViewState extends State<NumberFilterView> {
  Color colorPrimary, colorError, colorTextTertiary;
  final textController = AdeptTextEditingController();
  String lastText;
  List<DropdownMenuItem<String>> intFilterActions;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    onFilterChanged();
    generateActionItems();
    lastText = textController.text;
  }

  void generateActionItems(){
    intFilterActions = [
      DropdownMenuItem(
        value: null,
        child: Text(
          widget.textUnspecified,
          maxLines: 1,
          overflow: TextOverflow.ellipsis,
        ),
      ),
      DropdownMenuItem(
        value: FilterActions.EQUALS,
        child: Text("="),
      ),
      DropdownMenuItem(
        value: FilterActions.GREATER_THAN,
        child: Text(">"),
      ),
      DropdownMenuItem(
        value: FilterActions.GREATER_OR_EQUAL_THAN,
        child: Text(">="),
      ),
      DropdownMenuItem(
        value: FilterActions.LESS_THAN,
        child: Text("<"),
      ),
      DropdownMenuItem(
        value: FilterActions.LESS_OR_EQUAL_THAN,
        child: Text("<=",),
      ),
    ];
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
    textController.addListener(onTextChanged);
  }

  void onFilterChanged(){
    NumberFilter filter = widget.filterController.value;
    if(filter != null && filter.value != null){
      textController.text = "${filter.value}";
    }
  }

  void onTextChanged(){
    if(TextUtils.isEmpty(textController.text)){
      widget.filterController.value.value = null;
      return;
    }
    int value = int.tryParse(textController.text);
    if(value != null){
      widget.filterController.value.action(value, widget.filterController.value.paramAction);
      if(widget.filterController.error != null){
        widget.filterController.clearError();
        setState(() {});
      }
    } else{
      bool hadError = widget.filterController.error != null;
      widget.filterController.value.value = null;
      if(!hadError) {
        widget.filterController.error = widget.textExceptionValueValidation;
        setState(() {});
      }
    }
  }

  void checkColors(context){
    colorPrimary ??= Theme.of(context).primaryColor;
    colorError ??= Theme.of(context).errorColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Text(
            widget.hint,
            style: TextStyle(color: widget.filterController.value.paramAction != null ? colorPrimary : colorTextTertiary, fontSize: 12),
          ),
          widget.filterController.error == null ? Container() :
          Container(
            child: Text(
              widget.filterController.error,
              style: TextStyle(color: colorError, fontSize: 12),
            ),
          ),
          Row(
            children: <Widget>[
              Expanded(
                child: Theme(
                  data: ThemeData.light(),
                  child: DropdownButton<String>(
                    isExpanded: widget.filterController.value.paramAction == null,
                    items: intFilterActions,
                    value: widget.filterController.value.paramAction,
                    onChanged: (action){
                      if(action == null) {
                        widget.filterController.reset();
                      } else{
                        int value = int.tryParse(textController.text);
                        widget.filterController.value.action(value, action);
                      }
                      setState(() {});
                    },
                  ),
                ),
              ),
              widget.filterController.value.paramAction == null ? Container() :
              Expanded(
                child: Container(
                  child: TextField(
                    controller: textController,
                    textInputAction: TextInputAction.done,
                    keyboardType: TextInputType.number,
                    decoration: InputDecoration(
                      contentPadding: EdgeInsets.all(8),
                      focusedBorder: OutlineInputBorder(
                        borderSide: BorderSide(
                            width: 1,
                            color: Colors.grey.shade300
                        ),
                      ),
                      enabledBorder: OutlineInputBorder(
                        borderSide: BorderSide(
                            width: 1,
                            color: Colors.grey.shade300
                        ),
                      ),
                    ),
                  ),
                ),
              ),
              Container(
                margin: EdgeInsets.only(top: 16),
                alignment: Alignment.topCenter,
                child: SortView(
                  sortMode: widget.filterController.value.sortMode,
                  listener: (oldMode, newMode){
                    widget.filterController.value.sortMode = newMode;
                  },
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    super.dispose();
  }

}